const uint8_t PURE_S_ICONR[5]        = {0x49, 0x63, 0x6f, 0x6e, 0x0d};
const uint8_t PURE_S_RAR[6]          = {0x52, 0x61, 0x72, 0x21, 0x1a, 0x07};
const uint8_t PURE_S_TAR[5]          = {0x75, 0x73, 0x74, 0x61, 0x72};
const uint8_t PURE_S_XAR[4]          = {0x78, 0x61, 0x72, 0x21};
const uint8_t PURE_S_ZIP_CDH[4]      = {0x50, 0x4b, 0x01, 0x02};
const uint8_t PURE_S_ZIP_DDR[4]      = {0x50, 0x4b, 0x07, 0x08};
const uint8_t PURE_S_ZIP_EOCDL_64[4] = {0x50, 0x4b, 0x06, 0x07};
const uint8_t PURE_S_ZIP_EOCDR_64[4] = {0x50, 0x4b, 0x06, 0x06};
const uint8_t PURE_S_ZIP_EOCDR[4]    = {0x50, 0x4b, 0x05, 0x06};
const uint8_t PURE_S_ZIP_LFH[4]      = {0x50, 0x4b, 0x03, 0x04};
const uint8_t PURE_S_ZIP_PK[2]       = {0x50, 0x4b};
const uint8_t PURE_S_ZIP_SPAN[4]     = {0x50, 0x4b, 0x07, 0x08};
const uint8_t PURE_S_ZIP_TEMP[4]     = {0x50, 0x4b, 0x30, 0x30};

const uint64_t PURE_L_ICONR          = 5;
const uint64_t PURE_L_RAR            = 6;
const uint64_t PURE_L_TAR            = 5;
const uint64_t PURE_L_XAR            = 4;
const uint64_t PURE_L_ZIP_CDH        = 4;
const uint64_t PURE_L_ZIP_DDR        = 4;
const uint64_t PURE_L_ZIP_EOCDL_64   = 4;
const uint64_t PURE_L_ZIP_EOCDR_64   = 4;
const uint64_t PURE_L_ZIP_EOCDR      = 4;
const uint64_t PURE_L_ZIP_LFH        = 4;
const uint64_t PURE_L_ZIP_PK         = 2;
const uint64_t PURE_L_ZIP_SPAN       = 4;
const uint64_t PURE_L_ZIP_TEMP       = 4;